#include "magic/spells_info.h"

#include "magic/spells.h"
#include "utils/utils.h"

const char *unused_spellname = "!UNUSED!";

struct spellInfo_t spell_info[SPELLS_COUNT + 1];
struct spell_create_type spell_create[SPELLS_COUNT + 1];

void initUnusedSpell(int spl);
void initSpell(int spl, const char *name, const char *syn,
			   int max_mana, int min_mana, int mana_change,
			   int minpos, int targets, int violent, int routines, int danger, int spell_class);
void initSpells(void);
const char *spell_name(int num);

const char *spell_name(int num) {
	if (num > 0 && num <= SPELLS_COUNT) {
		return (spell_info[num].name);
	} else {
		if (num == -1) {
			return unused_spellname;
		} else {
			return "UNDEFINED";
		}
	}
}

void initUnusedSpell(int spl) {
	int i, j;
	for (i = 0; i < NUM_PLAYER_CLASSES; i++) {
		for (j = 0; j < NUM_KIN; j++) {
			spell_info[spl].min_remort[i][j] = MAX_REMORT;
			spell_info[spl].min_level[i][j] = LVL_IMPL + 1;
			spell_info[spl].slot_forc[i][j] = MAX_SLOT;
			spell_info[spl].class_change[i][j] = 0;
		}
	}

	for (i = 0; i < 3; i++) {
		spell_create[spl].wand.items[i] = -1;
		spell_create[spl].scroll.items[i] = -1;
		spell_create[spl].potion.items[i] = -1;
		spell_create[spl].items.items[i] = -1;
		spell_create[spl].runes.items[i] = -1;
	}

	spell_create[spl].wand.rnumber = -1;
	spell_create[spl].scroll.rnumber = -1;
	spell_create[spl].potion.rnumber = -1;
	spell_create[spl].items.rnumber = -1;
	spell_create[spl].runes.rnumber = -1;

	spell_info[spl].mana_max = 0;
	spell_info[spl].mana_min = 0;
	spell_info[spl].mana_change = 0;
	spell_info[spl].min_position = 0;
	spell_info[spl].danger = 0;
	spell_info[spl].targets = 0;
	spell_info[spl].violent = 0;
	spell_info[spl].routines = 0;
	spell_info[spl].name = unused_spellname;
	spell_info[spl].syn = unused_spellname;
}

void initSpell(int spl, const char *name, const char *syn,
			   int max_mana, int min_mana, int mana_change,
			   int minpos, int targets, int violent, int routines, int danger, int spell_class) {

	int i, j;
	for (i = 0; i < NUM_PLAYER_CLASSES; i++) {
		for (j = 0; j < NUM_KIN; j++) {
			spell_info[spl].min_remort[i][j] = MAX_REMORT;
			spell_info[spl].min_level[i][j] = LVL_IMPL;
			spell_info[spl].slot_forc[i][j] = MAX_SLOT;
			spell_info[spl].class_change[i][j] = 0;
		}
	}

	spell_create[spl].wand.min_caster_level = LVL_GRGOD;
	spell_create[spl].scroll.min_caster_level = LVL_GRGOD;
	spell_create[spl].potion.min_caster_level = LVL_GRGOD;
	spell_create[spl].items.min_caster_level = LVL_GRGOD;
	spell_create[spl].runes.min_caster_level = LVL_GRGOD;

	spell_info[spl].mana_max = max_mana;
	spell_info[spl].mana_min = min_mana;
	spell_info[spl].mana_change = mana_change;
	spell_info[spl].min_position = minpos;
	spell_info[spl].danger = danger;
	spell_info[spl].targets = targets;
	spell_info[spl].violent = violent;
	spell_info[spl].routines = routines;
	spell_info[spl].name = name;
	spell_info[spl].syn = syn;
	spell_info[spl].spell_class = spell_class;
}

/*
 * Arguments for initSpell calls:
 *
 * spellnum, maxmana, minmana, manachng, minpos, targets, violent?, routines.
 *
 * spellnum:  Number of the spell.  Usually the symbolic name as defined in
 * spells.h (such as SPELL_HEAL).
 *
 * spellname: The name of the spell.
 *
 * maxmana :  The maximum mana this spell will take (i.e., the mana it
 * will take when the player first gets the spell).
 *
 * minmana :  The minimum mana this spell will take, no matter how high
 * level the caster is.
 *
 * manachng:  The change in mana for the spell from level to level.  This
 * number should be positive, but represents the reduction in mana cost as
 * the caster's level increases.
 *
 * minpos  :  Minimum position the caster must be in for the spell to work
 * (usually fighting or standing). targets :  A "list" of the valid targets
 * for the spell, joined with bitwise OR ('|').
 *
 * violent :  TRUE or FALSE, depending on if this is considered a violent
 * spell and should not be cast in PEACEFUL rooms or on yourself.  Should be
 * set on any spell that inflicts damage, is considered aggressive (i.e.
 * charm, curse), or is otherwise nasty.
 *
 * routines:  A list of magic routines which are associated with this spell
 * if the spell uses spell templates.  Also joined with bitwise OR ('|').
 *
 * remort:  minimum number of remorts to use the spell.
 *
 * See the CircleMUD documentation for a more detailed description of these
 * fields.
 */

void initSpells(void) {

	for (int i = 0; i <= SPELLS_COUNT; i++) {
		initUnusedSpell(i);
	}


//1
	initSpell(SPELL_ARMOR, "", "armor", 40, 30, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);
//2
	initSpell(SPELL_TELEPORT, "", "teleport",
			  140, 120, 2, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_MANUAL | NPC_DAMAGE_PC, 1, STYPE_AIR);
//3
	initSpell(SPELL_BLESS, "", "bless", 55, 40, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF | TAR_OBJ_INV | TAR_OBJ_EQUIP,
			  FALSE, MAG_AFFECTS | MAG_ALTER_OBJS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);
//4
	initSpell(SPELL_BLINDNESS, "", "blind",
			  70, 40, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_DARK);
//5
	initSpell(SPELL_BURNING_HANDS,
			  " ",
			  "burning hands",
			  40,
			  30,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | NPC_DAMAGE_PC,
			  1,
			  STYPE_FIRE);
//6
	initSpell(SPELL_CALL_LIGHTNING,
			  " ",
			  "call lightning",
			  85,
			  70,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  NPC_AFFECT_PC | MAG_AFFECTS | MAG_DAMAGE | NPC_DAMAGE_PC,
			  2,
			  STYPE_AIR);
//7
	initSpell(SPELL_CHARM, " ", "mind control", 55, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_NOT_SELF, MTYPE_NEUTRAL, MAG_MANUAL, 1, STYPE_MIND);
//8
	initSpell(SPELL_CHILL_TOUCH, " ", "chill touch",
			  55, 45, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC, 1, STYPE_WATER);
//9
	initSpell(SPELL_CLONE, "", "clone",
			  150, 130, 5, POS_STANDING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_SUMMONS, 0, STYPE_DARK);
//10
	initSpell(SPELL_COLOR_SPRAY,
			  " ",
			  "ice bolts",
			  90,
			  75,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  3,
			  STYPE_WATER);
//11
	initSpell(SPELL_CONTROL_WEATHER, " ", "weather control",
			  100, 90, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL, 0, STYPE_AIR);
//12
	initSpell(SPELL_CREATE_FOOD, " ", "create food",
			  40, 30, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_CREATIONS, 0, STYPE_LIFE);
//13
	initSpell(SPELL_CREATE_WATER, " ", "create water", 40, 30, 1,
			  POS_STANDING, TAR_OBJ_INV | TAR_OBJ_EQUIP | TAR_CHAR_ROOM, FALSE, MAG_MANUAL, 0, STYPE_WATER);
//14
	initSpell(SPELL_CURE_BLIND, " ", "cure blind", 110, 90, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS | NPC_UNAFFECT_NPC, 0, STYPE_LIGHT);
//15
	initSpell(SPELL_CURE_CRITIC, " ", "critical cure",
			  100, 90, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_POINTS | NPC_DUMMY, 3, STYPE_LIFE);
//16
	initSpell(SPELL_CURE_LIGHT, " ", "light cure",
			  40, 30, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_POINTS | NPC_DUMMY, 1, STYPE_LIFE);
//17
	initSpell(SPELL_CURSE, "", "curse", 55, 40, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT | TAR_OBJ_INV, MTYPE_NEUTRAL,
			  MAG_AFFECTS | MAG_ALTER_OBJS | NPC_AFFECT_PC, 1, STYPE_DARK);
//18
	initSpell(SPELL_DETECT_ALIGN, " ", "detect alignment",
			  40, 30, 1, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//19
	initSpell(SPELL_DETECT_INVIS, " ", "detect invisible",
			  100, 55, 3, POS_FIGHTING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//20
	initSpell(SPELL_DETECT_MAGIC, " ", "detect magic",
			  100, 55, 3, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//21
	initSpell(SPELL_DETECT_POISON, " ", "detect poison",
			  40, 30, 1, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//22
	initSpell(SPELL_DISPEL_EVIL, " ", "dispel evil",
			  100, 90, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE, 1, STYPE_LIGHT);
//23
	initSpell(SPELL_EARTHQUAKE, "", "earthquake", 110, 90, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_DAMAGE | NPC_DAMAGE_PC, 2, STYPE_EARTH);
//24
	initSpell(SPELL_ENCHANT_WEAPON, " ", "enchant weapon",
			  140, 110, 2, POS_STANDING, TAR_OBJ_INV, FALSE, MAG_ALTER_OBJS, 0, STYPE_LIGHT);
//25
	initSpell(SPELL_ENERGY_DRAIN, " ", "energy drain",
			  150, 140, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_MANUAL | MAG_AFFECTS | NPC_DAMAGE_PC, 1, STYPE_DARK);
//26
	initSpell(SPELL_FIREBALL, " ", "fireball",
			  110, 100, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE, MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  2, STYPE_FIRE);
//27
	initSpell(SPELL_HARM, "", "harm",
			  110, 100, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 5, STYPE_DARK);
//28
	initSpell(SPELL_HEAL, "", "heal", 110, 100, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_POINTS | NPC_DUMMY, 10, STYPE_LIFE);
//29
	initSpell(SPELL_INVISIBLE, "", "invisible",
			  50, 40, 3, POS_STANDING, TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_ROOM,
			  FALSE, MAG_AFFECTS | MAG_ALTER_OBJS, 0, STYPE_MIND);
//30
	initSpell(SPELL_LIGHTNING_BOLT, "", "lightning bolt", 55, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE, MAG_DAMAGE | NPC_DAMAGE_PC, 1, STYPE_AIR);
//31
	initSpell(SPELL_LOCATE_OBJECT, " ", "locate object",
			  140, 110, 2, POS_STANDING, TAR_OBJ_WORLD, FALSE, MAG_MANUAL, 0, STYPE_MIND);
//32
	initSpell(SPELL_MAGIC_MISSILE, " ", "magic missle",
			  40, 30, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 1, STYPE_FIRE);
//33
	initSpell(SPELL_POISON, "", "poison", 70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_OBJ_INV | TAR_FIGHT_VICT,
			  MTYPE_NEUTRAL, MAG_AFFECTS | MAG_ALTER_OBJS | NPC_AFFECT_PC, 2, STYPE_LIFE);
//34
	initSpell(SPELL_PROT_FROM_EVIL, "  ", "protect evil", 60, 45, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);
//35
	initSpell(SPELL_REMOVE_CURSE, " ", "remove curse",
			  50, 40, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF | TAR_OBJ_INV | TAR_OBJ_EQUIP, FALSE,
			  MAG_UNAFFECTS | MAG_ALTER_OBJS | NPC_UNAFFECT_NPC, 0, STYPE_LIGHT);
//36
	initSpell(SPELL_SANCTUARY, "", "sanctuary", 85, 70, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);
//37
	initSpell(SPELL_SHOCKING_GRASP,
			  " ",
			  "shocking grasp",
			  50,
			  40,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_DAMAGE_PC,
			  1,
			  STYPE_FIRE);
//38
	initSpell(SPELL_SLEEP, "", "sleep",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 0, STYPE_MIND);
//39
	initSpell(SPELL_STRENGTH, "", "strength", 40, 30, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//40
	initSpell(SPELL_SUMMON, "", "summon",
			  110, 100, 2, POS_STANDING, TAR_CHAR_WORLD | TAR_NOT_SELF, FALSE, MAG_MANUAL, 0, STYPE_MIND);
//41
	initSpell(SPELL_PATRONAGE, "", "patronage", 85, 70, 2,
			  POS_FIGHTING, TAR_SELF_ONLY | TAR_CHAR_ROOM, FALSE, MAG_POINTS | MAG_AFFECTS, 1, STYPE_LIGHT);
//42
	initSpell(SPELL_WORD_OF_RECALL, " ", "recall", 140, 100, 4,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_MANUAL | NPC_DAMAGE_PC, 0, STYPE_MIND);
//43
	initSpell(SPELL_REMOVE_POISON, " ", "remove poison",
			  60, 45, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF | TAR_OBJ_INV | TAR_OBJ_ROOM, FALSE,
			  MAG_UNAFFECTS | MAG_ALTER_OBJS | NPC_UNAFFECT_NPC, 0, STYPE_LIFE);
//44
	initSpell(SPELL_SENSE_LIFE, " ", "sense life",
			  85, 70, 1, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_LIFE);
//45
	initSpell(SPELL_ANIMATE_DEAD, " ", "animate dead",
			  50, 35, 3, POS_STANDING, TAR_OBJ_ROOM, FALSE, MAG_SUMMONS, 0, STYPE_DARK);
//46
	initSpell(SPELL_DISPEL_GOOD, " ", "dispel good", 100, 90, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE, 1, STYPE_DARK);
//47
	initSpell(SPELL_GROUP_ARMOR, " ", "group armor", 110, 100, 2,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);
//48
	initSpell(SPELL_GROUP_HEAL, " ", "group heal",
			  110, 100, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_POINTS | NPC_DUMMY, 30, STYPE_LIFE);
//49
	initSpell(SPELL_GROUP_RECALL, " ", "group recall",
			  125, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_MANUAL, 0, STYPE_MIND);
//50
	initSpell(SPELL_INFRAVISION, " ", "infravision",
			  50, 40, 2, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_LIGHT);
//51
	initSpell(SPELL_WATERWALK, "", "waterwalk",
			  70, 55, 1, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_WATER);
//52
	initSpell(SPELL_CURE_SERIOUS, " ", "serious cure", 85, 70, 4,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_POINTS | NPC_DUMMY, 2, STYPE_LIFE);
//53
	initSpell(SPELL_GROUP_STRENGTH, " ", "group strength", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//54
	initSpell(SPELL_HOLD, "", "hold",
			  100, 40, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 3, STYPE_MIND);
//55
	initSpell(SPELL_POWER_HOLD, " ", "power hold",
			  140, 90, 4, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 4, STYPE_MIND);
//56
	initSpell(SPELL_MASS_HOLD, " ", "mass hold",
			  150, 130, 5, POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC,
			  5, STYPE_MIND);
//57
	initSpell(SPELL_FLY, "", "fly", 50, 35, 1, POS_STANDING,
			  TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_EQUIP,
			  FALSE, MAG_AFFECTS | MAG_ALTER_OBJS, 0, STYPE_AIR);
//58
	initSpell(SPELL_BROKEN_CHAINS, " ", "broken chains", 125, 110, 2,
			  POS_FIGHTING, TAR_SELF_ONLY | TAR_CHAR_ROOM, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 5, STYPE_MIND);

//59
	initSpell(SPELL_NOFLEE, " ", "noflee",
			  100, 90, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 0, STYPE_MIND);
//60
	initSpell(SPELL_CREATE_LIGHT, " ", "create light",
			  40, 30, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_CREATIONS, 0, STYPE_LIGHT);
//61
	initSpell(SPELL_DARKNESS, "", "darkness", 100, 70, 2, POS_STANDING,
			  TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_EQUIP,
			  FALSE, MAG_AFFECTS | MAG_ALTER_OBJS, 0, STYPE_DARK);
//62
	initSpell(SPELL_STONESKIN, " ", "stoneskin", 55, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_EARTH);
//63
	initSpell(SPELL_CLOUDLY, "", "cloudly", 55, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_WATER);
//64
	initSpell(SPELL_SILENCE, "", "sielence",
			  100, 40, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL,
			  MAG_AFFECTS | NPC_AFFECT_PC | NPC_AFFECT_PC_CASTER, 1, STYPE_MIND);
//65
	initSpell(SPELL_LIGHT, "", "sun shine", 100, 70, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_EQUIP,
			  FALSE, MAG_AFFECTS | MAG_ALTER_OBJS, 0, STYPE_LIGHT);
//66
	initSpell(SPELL_CHAIN_LIGHTNING, " ", "chain lightning",
			  120, 110, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 1, STYPE_AIR);
//67
	initSpell(SPELL_FIREBLAST, " ", "fireblast",
			  110, 90, 2, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_DAMAGE | NPC_DAMAGE_PC,
			  5, STYPE_FIRE);
//68
	initSpell(SPELL_IMPLOSION, " ", "implosion",
			  140, 120, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE, MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  15, STYPE_FIRE);
//69
	initSpell(SPELL_WEAKNESS, "", "weakness",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 0, STYPE_LIFE);
//70
	initSpell(SPELL_GROUP_INVISIBLE, " ", "group invisible",
			  150, 130, 5, POS_STANDING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS, 0, STYPE_MIND);
//71
	initSpell(SPELL_SHADOW_CLOAK, " ", "shadow cloak", 100, 70, 3,
			  POS_FIGHTING, TAR_SELF_ONLY | TAR_CHAR_ROOM, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_DARK);
//72
	initSpell(SPELL_ACID,
			  "",
			  "acid",
			  90,
			  65,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_NEUTRAL,
			  MAG_DAMAGE | MAG_ALTER_OBJS | NPC_DAMAGE_PC,
			  2,
			  STYPE_WATER);
//73
	initSpell(SPELL_REPAIR, "", "repair",
			  110, 100, 1, POS_STANDING, TAR_OBJ_INV | TAR_OBJ_EQUIP, FALSE, MAG_ALTER_OBJS, 0, STYPE_LIGHT);
//74
	initSpell(SPELL_ENLARGE, "", "enlarge",
			  55, 40, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC,
			  0, STYPE_LIFE);
//75
	initSpell(SPELL_FEAR, "", "fear",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_MANUAL | NPC_DAMAGE_PC, 1, STYPE_DARK);
//76
	initSpell(SPELL_SACRIFICE, " ", "sacrifice",
			  140, 125, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_MANUAL | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  10, STYPE_DARK);
//77
	initSpell(SPELL_WEB, "", "web",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_MIND);
//78
	initSpell(SPELL_BLINK, "", "blink", 70, 55, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);
//79
	initSpell(SPELL_REMOVE_HOLD, " ", "remove hold", 110, 90, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS | NPC_UNAFFECT_NPC, 1, STYPE_LIGHT);
//80
	initSpell(SPELL_CAMOUFLAGE, "!!", "!set by skill!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//81
	initSpell(SPELL_POWER_BLINDNESS, " ", "power blind",
			  110, 100, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_DARK);
//82
	initSpell(SPELL_MASS_BLINDNESS, " ", "mass blind", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 4, STYPE_DARK);
//83
	initSpell(SPELL_POWER_SILENCE, " ", "power sielence",
			  120, 90, 4, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL,
			  MAG_AFFECTS | NPC_AFFECT_PC | NPC_AFFECT_PC_CASTER, 2, STYPE_MIND);
//84
	initSpell(SPELL_EXTRA_HITS, " ", "extra hits",
			  100, 85, 2, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_POINTS | NPC_DUMMY, 1, STYPE_LIFE);
//85
	initSpell(SPELL_RESSURECTION, " ", "ressurection",
			  120, 100, 2, POS_STANDING, TAR_OBJ_ROOM, FALSE, MAG_SUMMONS, 0, STYPE_DARK);
//86
	initSpell(SPELL_MAGICSHIELD, " ", "magic shield", 50, 30, 2,
			  POS_FIGHTING, TAR_SELF_ONLY | TAR_CHAR_ROOM, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIGHT);

//87
	initSpell(SPELL_FORBIDDEN, " ", "forbidden",
			  125, 110, 2, POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_ROOM, 0, STYPE_MIND);
//88
	initSpell(SPELL_MASS_SILENCE, " ", "mass sielence", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 3, STYPE_MIND);
//89
	initSpell(SPELL_REMOVE_SILENCE, " ", "remove sielence",
			  70, 55, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS | NPC_UNAFFECT_NPC | NPC_UNAFFECT_NPC_CASTER,
			  1, STYPE_LIGHT);
//90
	initSpell(SPELL_DAMAGE_LIGHT, " ", "light damage",
			  40, 30, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 1, STYPE_DARK);
//91
	initSpell(SPELL_DAMAGE_SERIOUS, " ", "serious damage",
			  85, 55, 4, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 2, STYPE_DARK);
//92
	initSpell(SPELL_DAMAGE_CRITIC, " ", "critical damage",
			  100, 90, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 3, STYPE_DARK);
//93
	initSpell(SPELL_MASS_CURSE, " ", "mass curse", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_DARK);
//94
	initSpell(SPELL_ARMAGEDDON, " ", "armageddon", 150, 130, 5,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_DAMAGE | NPC_DAMAGE_PC, 10, STYPE_AIR);
//95
	initSpell(SPELL_GROUP_FLY, " ", "group fly",
			  140, 120, 2, POS_STANDING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS, 0, STYPE_AIR);
//96
	initSpell(SPELL_GROUP_BLESS, " ", "group bless", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);
//97
	initSpell(SPELL_REFRESH, "", "refresh",
			  80, 60, 1, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_POINTS, 0, STYPE_LIFE);
//98
	initSpell(SPELL_STUNNING, " ", "stunning", 150, 140, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE, 15, STYPE_EARTH);

//99
	initSpell(SPELL_HIDE, "!!", "!set by skill!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//100
	initSpell(SPELL_SNEAK, "!!", "!set by skill!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//101
	initSpell(SPELL_DRUNKED, "!!", "!set by programm!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//102
	initSpell(SPELL_ABSTINENT, "!!", "!set by programm!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//103
	initSpell(SPELL_FULL, "", "full", 70, 55, 1,
			  POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_POINTS, 10, STYPE_LIFE);
//104
	initSpell(SPELL_CONE_OF_COLD,
			  " ",
			  "cold wind",
			  100,
			  90,
			  1,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  MAG_AFFECTS | MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  15,
			  STYPE_WATER);
//105
	initSpell(SPELL_BATTLE, "!  !", "!set by programm!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//106
	initSpell(SPELL_HAEMORRAGIA, "!!", "!set by programm!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//107
	initSpell(SPELL_COURAGE, "!!", "!set by programm!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//108
	initSpell(SPELL_WATERBREATH, " ", "waterbreath",
			  85, 70, 4, POS_STANDING, TAR_CHAR_ROOM, FALSE, MAG_AFFECTS, 0, STYPE_WATER);
//109
	initSpell(SPELL_SLOW, "", "slow",
			  55, 40, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_MIND);
//110
	initSpell(SPELL_HASTE, "", "haste", 55, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//111
	initSpell(SPELL_MASS_SLOW, " ", "mass slow", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_MIND);
//112
	initSpell(SPELL_GROUP_HASTE, " ", "group haste", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_MIND);
//113
	initSpell(SPELL_SHIELD, " ", "gods shield", 150, 140, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 2, STYPE_LIGHT);
//114
	initSpell(SPELL_PLAQUE, "", "plaque",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_LIFE);
//115
	initSpell(SPELL_CURE_PLAQUE, " ", "cure plaque", 85, 70, 4,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS | NPC_UNAFFECT_NPC, 0, STYPE_LIFE);
//116
	initSpell(SPELL_AWARNESS, "", "awarness", 100, 90, 1,
			  POS_STANDING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//117
	initSpell(SPELL_RELIGION, "!  !", "!pray or donate!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//118
	initSpell(SPELL_AIR_SHIELD, " ", "air shield", 140, 120, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_AIR);
//119
	initSpell(SPELL_PORTAL, "", "portal", 200, 180, 4,
			  POS_STANDING, TAR_CHAR_WORLD, FALSE, MAG_MANUAL, 0, STYPE_LIGHT);
//120
	initSpell(SPELL_DISPELL_MAGIC, " ", "dispel magic",
			  85, 70, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS, 0, STYPE_LIGHT);
//121
	initSpell(SPELL_SUMMON_KEEPER, "", "keeper",
			  100, 80, 2, POS_STANDING, TAR_IGNORE, FALSE, MAG_SUMMONS, 0, STYPE_LIGHT);
//122
	initSpell(SPELL_FAST_REGENERATION, " ",
			  "fast regeneration", 100, 90, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//123
	initSpell(SPELL_CREATE_WEAPON, " ", "create weapon",
			  130, 110, 2, POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL, 0, STYPE_LIGHT);
//124
	initSpell(SPELL_FIRE_SHIELD, " ", "fire shield", 140, 120, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_FIRE);
//125
	initSpell(SPELL_RELOCATE, "", "relocate",
			  140, 120, 2, POS_STANDING, TAR_CHAR_WORLD, FALSE, MAG_MANUAL, 0, STYPE_AIR);
//126
	initSpell(SPELL_SUMMON_FIREKEEPER, " ", "fire keeper",
			  150, 140, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_SUMMONS, 0, STYPE_FIRE);
//127
	initSpell(SPELL_ICE_SHIELD, " ", "ice protect", 140, 120, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_WATER);
//128
	initSpell(SPELL_ICESTORM, " ", "ice storm",
			  125, 110, 2, POS_FIGHTING,
			  TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC, 5, STYPE_WATER);
//129
	initSpell(SPELL_ENLESS, "", "enless",
			  55, 40, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS, 0, STYPE_LIFE);
//130
	initSpell(SPELL_SHINEFLASH, " ", "shine flash",
			  60, 45, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_FIRE);
//131
	initSpell(SPELL_MADNESS, "", "madness",
			  130, 110, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_MIND);
//132
	initSpell(SPELL_GROUP_MAGICGLASS, " ", "group magicglass",
			  140, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 4, STYPE_AIR);
//133
	initSpell(SPELL_CLOUD_OF_ARROWS, " ", "cloud of arrous", 95, 80, 2,
			  POS_FIGHTING, TAR_SELF_ONLY | TAR_CHAR_ROOM, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 4, STYPE_FIRE);
//134
	initSpell(SPELL_VACUUM, " ", "vacuum sphere",
			  150, 140, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_DAMAGE | NPC_DAMAGE_PC, 15, STYPE_DARK);
//135
	initSpell(SPELL_METEORSTORM, " ", "meteor storm", 125, 110, 2,
			  POS_FIGHTING, TAR_ROOM_THIS, FALSE, MAG_NEED_CONTROL | MAG_ROOM | MAG_CASTER_INROOM, 0, STYPE_EARTH);
//136
	initSpell(SPELL_STONEHAND, " ", "stonehand", 40, 30, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_EARTH);
//137
	initSpell(SPELL_MINDLESS, " ", "mindness",
			  120, 110, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC | NPC_AFFECT_PC_CASTER,
			  0, STYPE_MIND);
//138
	initSpell(SPELL_PRISMATICAURA, " ", "prismatic aura", 85, 70, 4,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);
//139
	initSpell(SPELL_EVILESS, " ", "eviless", 150, 130, 5,
			  POS_STANDING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | MAG_POINTS, 3, STYPE_DARK);
//140
	initSpell(SPELL_AIR_AURA, " ", "air aura",
			  140, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_AIR);
//141
	initSpell(SPELL_FIRE_AURA, " ", "fire aura",
			  140, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_FIRE);
//142
	initSpell(SPELL_ICE_AURA, " ", "ice aura",
			  140, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_WATER);
//143
	initSpell(SPELL_SHOCK, "", "shock", 100, 90, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 1, STYPE_DARK);
//144
	initSpell(SPELL_MAGICGLASS, " ", "magic glassie", 120, 110, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 2, STYPE_LIGHT);

//145
	initSpell(SPELL_GROUP_SANCTUARY, " ", "group sanctuary", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);

//146
	initSpell(SPELL_GROUP_PRISMATICAURA, "  ",
			  "group prismatic aura", 110, 100, 1, POS_FIGHTING, TAR_IGNORE,
			  FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);

//147
	initSpell(SPELL_DEAFNESS, "", "deafness",
			  100, 40, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL,
			  MAG_AFFECTS | NPC_AFFECT_PC | NPC_AFFECT_PC_CASTER, 1, STYPE_MIND);

//148
	initSpell(SPELL_POWER_DEAFNESS, " ", "power deafness",
			  120, 90, 4, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL,
			  MAG_AFFECTS | NPC_AFFECT_PC | NPC_AFFECT_PC_CASTER, 2, STYPE_MIND);

//149
	initSpell(SPELL_REMOVE_DEAFNESS, " ", "remove deafness",
			  90, 80, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_UNAFFECTS | NPC_UNAFFECT_NPC | NPC_UNAFFECT_NPC_CASTER,
			  1, STYPE_LIFE);

//150
	initSpell(SPELL_MASS_DEAFNESS, " ", "mass deafness", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_MIND);

//151
	initSpell(SPELL_DUSTSTORM, " ", "dust storm",
			  125, 110, 2, POS_FIGHTING,
			  TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC, 5, STYPE_EARTH);

//152
	initSpell(SPELL_EARTHFALL, "", "earth fall",
			  120, 110, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 1, STYPE_EARTH);

//153
	initSpell(SPELL_SONICWAVE, " ", "sonic wave",
			  120, 110, 2, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE,
			  MAG_MASSES | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 1, STYPE_AIR);

//154
	initSpell(SPELL_HOLYSTRIKE, " ", "holystrike",
			  150, 130, 5, POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MANUAL | NPC_DAMAGE_PC, 10, STYPE_LIGHT);

//155
	initSpell(SPELL_ANGEL, "-", "angel", 150, 130, 5,
			  POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL, 1, STYPE_LIGHT);
//156
	initSpell(SPELL_MASS_FEAR, " ", "mass fear", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_MANUAL | NPC_AFFECT_PC, 4, STYPE_DARK);
//157
	initSpell(SPELL_FASCINATION, "", "fascination", 480, 440, 20,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 2, STYPE_MIND);
//158
	initSpell(SPELL_CRYING, "", "crying",
			  120, 55, 2, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_MIND);
//159
	initSpell(SPELL_OBLIVION, "", "oblivion",
			  70, 55, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_DARK);
//160
	initSpell(SPELL_BURDEN_OF_TIME, " ", "burden time", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_AREAS | MAG_AFFECTS | NPC_AFFECT_PC, 4, STYPE_DARK);
//161
	initSpell(SPELL_GROUP_REFRESH, " ", "group refresh",    //Added by Adept
			  160, 140, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_POINTS | NPC_DUMMY, 30, STYPE_LIFE);

//162
	initSpell(SPELL_PEACEFUL, "", "peaceful",
			  55, 40, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_NOT_SELF | TAR_FIGHT_VICT, MTYPE_NEUTRAL, MAG_AFFECTS | NPC_AFFECT_PC, 1, STYPE_MIND);
//163
	initSpell(SPELL_MAGICBATTLE, "!  !", "!set by programm!", 0, 0, 0, 255, 0,
			  FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);

//164
	initSpell(SPELL_BERSERK, "! !", "!set by programm!", 0, 0, 0, 255, 0,
			  FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//165
	initSpell(SPELL_STONEBONES, " ", "stone bones", 80, 40, 1,
			  POS_STANDING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_EARTH);

//166 - SPELL_ROOM_LIGHT
	initSpell(SPELL_ROOM_LIGHT, " ", "room light", 10, 10, 1,
			  POS_STANDING, TAR_ROOM_THIS, FALSE, MAG_ROOM, 0, STYPE_LIGHT);

//167 - SPELL_POISONED_FOG
	initSpell(SPELL_POISONED_FOG, " ", "poisoned fog", 10, 10, 1,
			  POS_STANDING, TAR_ROOM_THIS, FALSE, MAG_ROOM | MAG_CASTER_INROOM, 0, STYPE_LIFE);

//168 - SPELL_THUNDERSTORM
	initSpell(SPELL_THUNDERSTORM, " ", "storm of vengeance", 10, 10, 1,
			  POS_STANDING, TAR_ROOM_THIS, FALSE, MAG_NEED_CONTROL | MAG_ROOM | MAG_CASTER_INROOM, 0, STYPE_AIR);
//169
	initSpell(SPELL_LIGHT_WALK, "! !", "!set by programm!", 0, 0, 0, 255, 0,
			  FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//170
	initSpell(SPELL_FAILURE, "", "failure", 100, 85, 2, POS_FIGHTING,
			  TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 5, STYPE_DARK);

//171
	initSpell(SPELL_CLANPRAY, "! !", "!clan affect!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//172
	initSpell(SPELL_GLITTERDUST, " ", "glitterdust", 120, 100, 3,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 5, STYPE_EARTH);
//173
	initSpell(SPELL_SCREAM, "", "scream", 100, 85, 3, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_AIR);
//174
	initSpell(SPELL_CATS_GRACE, " ", "cats grace", 50, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//175
	initSpell(SPELL_BULL_BODY, " ", "bull body", 50, 40, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//176
	initSpell(SPELL_SNAKE_WISDOM, " ", "snake wisdom", 60, 50, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//177
	initSpell(SPELL_GIMMICKRY, "", "gimmickry", 60, 50, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//    mana_change 
//     ,
//     
//178
/*
	initSpell(SPELL_WC_OF_CHALLENGE, " ", "warcry of challenge", 10, 10, 1,
		   POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
		   MAG_WARCRY | MAG_AREAS | MAG_DAMAGE | NPC_DAMAGE_PC, 0, STYPE_MIND);
//179
	initSpell(SPELL_WC_OF_MENACE, " ", "warcry of menace", 30, 30, 40,
		   POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
		   MAG_WARCRY | MAG_AREAS | MAG_AFFECTS | NPC_AFFECT_PC, 0, STYPE_MIND);
//180
	initSpell(SPELL_WC_OF_RAGE, " ", "warcry of rage", 30, 30, 50,
		   POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
		   MAG_WARCRY | MAG_AREAS | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC, 0, STYPE_MIND);
//181
	initSpell(SPELL_WC_OF_MADNESS, " ", "warcry of madness", 50, 50, 91,
		   POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
		   MAG_WARCRY | MAG_AREAS | MAG_AFFECTS | NPC_AFFECT_PC, 0, STYPE_MIND);
//182
	initSpell(SPELL_WC_OF_THUNDER, " ", "warcry of thunder", 140, 140, 141,
		   POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
		   MAG_WARCRY | MAG_AREAS | MAG_DAMAGE | MAG_AFFECTS | NPC_DAMAGE_PC, 0, STYPE_MIND);
*/
//183
	initSpell(SPELL_WC_OF_DEFENSE, " ", "warcry of defense", 10, 10, 10,
			  POS_FIGHTING, TAR_IGNORE, FALSE,
			  MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//184
	initSpell(SPELL_WC_OF_BATTLE, " ", "warcry of battle", 20, 20, 50,
			  POS_FIGHTING, TAR_IGNORE, FALSE,
			  MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//185
	initSpell(SPELL_WC_OF_POWER, " ", "warcry of power", 25, 25, 70,
			  POS_FIGHTING, TAR_IGNORE, FALSE,
			  MAG_WARCRY | MAG_GROUPS | MAG_POINTS | MAG_AFFECTS | NPC_DUMMY | NPC_AFFECT_NPC, 0, STYPE_MIND);
//186
	initSpell(SPELL_WC_OF_BLESS, " ", "warcry of bless", 15, 15, 30,
			  POS_FIGHTING, TAR_IGNORE, FALSE,
			  MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//187
	initSpell(SPELL_WC_OF_COURAGE, " ", "warcry of courage", 10, 10, 10,
			  POS_FIGHTING, TAR_IGNORE, FALSE,
			  MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//188
	initSpell(SPELL_RUNE_LABEL, " ", "rune label", 50, 35, 1,
			  POS_STANDING, TAR_ROOM_THIS, FALSE, MAG_ROOM | MAG_CASTER_INWORLD_DELAY, 0, STYPE_LIGHT);


	// NON-castable spells should appear below here.

// 189
	initSpell(SPELL_ACONITUM_POISON, " ", "aconitum poison",
			  0, 0, 0, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_AFFECTS, 0, STYPE_LIFE);
// 190
	initSpell(SPELL_SCOPOLIA_POISON, " ", "scopolia poison",
			  0, 0, 0, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_AFFECTS, 0, STYPE_LIFE);
// 191
	initSpell(SPELL_BELENA_POISON, " ", "belena poison",
			  0, 0, 0, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_AFFECTS, 0, STYPE_LIFE);
// 192
	initSpell(SPELL_DATURA_POISON, " ", "datura poison",
			  0, 0, 0, POS_FIGHTING, TAR_IGNORE, MTYPE_AGGRESSIVE, MAG_AFFECTS, 0, STYPE_LIFE);


// 193
	initSpell(SPELL_TIMER_REPAIR, " ", " timer repair",
			  110, 100, 1, POS_STANDING, TAR_OBJ_INV | TAR_OBJ_EQUIP, FALSE, MAG_ALTER_OBJS, 0, STYPE_LIGHT);

//194
	initSpell(SPELL_LACKY, " ", "lacky", 100, 90, 1,
			  POS_STANDING, TAR_CHAR_ROOM | TAR_SELF_ONLY, FALSE, MAG_AFFECTS, 0, STYPE_MIND);
//195
	initSpell(SPELL_BANDAGE, "", "!set by programm!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//196
	initSpell(SPELL_NO_BANDAGE,
			  "! !",
			  "!set by programm!",
			  0,
			  0,
			  0,
			  255,
			  0,
			  FALSE,
			  MAG_MANUAL,
			  0,
			  STYPE_NEUTRAL);
//197
	initSpell(SPELL_CAPABLE, "!!", "!set by programm!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//198
	initSpell(SPELL_STRANGLE, "!!", "!set by programm!", 0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//199
	initSpell(SPELL_RECALL_SPELLS, "! !", "!set by programm!",
		0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//200
	initSpell(SPELL_HYPNOTIC_PATTERN, " ", "hypnotic pattern", 120, 100, 2,
			  POS_STANDING, TAR_ROOM_THIS, FALSE, MAG_ROOM | MAG_CASTER_INROOM, 0, STYPE_MIND);
//201
	initSpell(SPELL_SOLOBONUS, "", "bonus",
		55, 40, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_NOT_SELF, MTYPE_NEUTRAL, MAG_MANUAL, 1, STYPE_NEUTRAL);
//202
	initSpell(SPELL_VAMPIRE, "", "vampire", 150, 130, 5, POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL,
			  3, STYPE_DARK);
//203
	initSpell(SPELLS_RESTORATION, "", "reconstruction",
			  110, 100, 1, POS_STANDING, TAR_OBJ_INV | TAR_OBJ_EQUIP, FALSE, MAG_ALTER_OBJS, 0, STYPE_LIGHT);
//204
	initSpell(SPELL_AURA_DEATH, " ", "aura death",
			  140, 120, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_DARK);
//205
	initSpell(SPELL_RECOVERY,
			  " ",
			  "recovery",
			  110,
			  100,
			  1,
			  POS_STANDING,
			  TAR_CHAR_ROOM | TAR_FIGHT_SELF,
			  FALSE,
			  MAG_AFFECTS | NPC_AFFECT_NPC,
			  0,
			  STYPE_DARK);
//206
	initSpell(SPELL_MASS_RECOVERY, " ", "mass recovery",
			  110, 100, 1, POS_STANDING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_POINTS | NPC_DUMMY, 0, STYPE_DARK);
//207
	initSpell(SPELL_AURA_EVIL, " ", "aura evil",
			  150, 130, 5, POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL, 3, STYPE_DARK);
//208
	initSpell(SPELL_MENTAL_SHADOW, " ", "mental shadow", 150, 130, 5,
			  POS_STANDING, TAR_IGNORE, FALSE, MAG_MANUAL, 1, STYPE_MIND);
//209
	initSpell(SPELL_EVARDS_BLACK_TENTACLES, " ", "evards black tentacles", 120, 110, 2,
			  POS_FIGHTING, TAR_ROOM_THIS, FALSE, MAG_NEED_CONTROL | MAG_ROOM | MAG_CASTER_INROOM, 0, STYPE_DARK);
//210
	initSpell(SPELL_WHIRLWIND, "", "whirlwind", 110, 100, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_AIR);
//211
	initSpell(SPELL_INDRIKS_TEETH, " ", "indriks teeth", 60, 45, 1, POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_EARTH);
//212
	initSpell(SPELL_MELFS_ACID_ARROW, " ", "acid arrow", 110, 100, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_WATER);
//213
	initSpell(SPELL_THUNDERSTONE, " ", "thunderstone", 110, 100, 2,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_EARTH);
//214
	initSpell(SPELL_CLOD, "", "clod", 110, 100, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_EARTH);
//215
	initSpell(SPELL_EXPEDIENT, "! !", "!set by programm!",
			  0, 0, 0, 255, 0, FALSE, MAG_MANUAL, 0, STYPE_NEUTRAL);
//216
	initSpell(SPELL_SIGHT_OF_DARKNESS, " ", "sight darkness", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);
//217
	initSpell(SPELL_GENERAL_SINCERITY, " ", "general sincerity", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_MIND);
//218
	initSpell(SPELL_MAGICAL_GAZE, " ", "magical gaze", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_MIND);
//219
	initSpell(SPELL_ALL_SEEING_EYE, " ", "allseeing eye", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_MIND);
//220
	initSpell(SPELL_EYE_OF_GODS, " ", "eye gods", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIFE);
//221
	initSpell(SPELL_BREATHING_AT_DEPTH, " ", "breathing at depth", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_WATER);
//222
	initSpell(SPELL_GENERAL_RECOVERY, " ", "general recovery", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIFE);
//223
	initSpell(SPELL_COMMON_MEAL, " ", "common meal", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_POINTS | NPC_DUMMY, 1, STYPE_LIFE);
//224
	initSpell(SPELL_STONE_WALL, " ", "stone wall", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_EARTH);
//225
	initSpell(SPELL_SNAKE_EYES, " ", "snake eyes", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_MIND);
//226
	initSpell(SPELL_EARTH_AURA, " ", "earth aura", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_EARTH);
//227
	initSpell(SPELL_GROUP_PROT_FROM_EVIL, "   ", "group protect evil", 110, 100, 1,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 1, STYPE_LIGHT);

//228
	initSpell(SPELL_ARROWS_FIRE, " ", "arrows of fire", 0, 0, 0,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_FIRE);
//229
	initSpell(SPELL_ARROWS_WATER, " ", "arrows of water", 0, 0, 0,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_WATER);
//230
	initSpell(SPELL_ARROWS_EARTH,
			  " ",
			  "arrows of earth",
			  0,
			  0,
			  0,
			  POS_FIGHTING,
			  TAR_CHAR_ROOM | TAR_FIGHT_VICT,
			  MTYPE_AGGRESSIVE,
			  MAG_AREAS | MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP,
			  2,
			  STYPE_EARTH);
//231
	initSpell(SPELL_ARROWS_AIR, " ", "arrows of air", 0, 0, 0,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_AIR);
//232
	initSpell(SPELL_ARROWS_DEATH, " ", "arrows of death", 0, 0, 0,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_VICT, MTYPE_AGGRESSIVE,
			  MAG_DAMAGE | NPC_AFFECT_PC | MAG_AFFECTS | NPC_DAMAGE_PC | NPC_DAMAGE_PC_MINHP, 2, STYPE_DARK);
//233
	initSpell(SPELL_PALADINE_INSPIRATION, "", "inspiration",
			  0, 0, 0, 255, 0, FALSE, MAG_GROUPS | MAG_AFFECTS, 0, STYPE_NEUTRAL);
//234
	initSpell(SPELL_DEXTERITY, "", "dexterity", 40, 30, 1,
			  POS_FIGHTING, TAR_CHAR_ROOM | TAR_FIGHT_SELF, FALSE, MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_LIFE);
//235
	initSpell(SPELL_GROUP_BLINK, " ", "group blink",
			  110, 100, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 30, STYPE_LIFE);
//236
	initSpell(SPELL_GROUP_CLOUDLY, " ", "group cloudly",
			  110, 100, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 30, STYPE_WATER);
//237
	initSpell(SPELL_GROUP_AWARNESS, " ", "group awarness",
			  110, 100, 2, POS_FIGHTING, TAR_IGNORE, FALSE, MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 30, STYPE_MIND);
//238
	initSpell(SPELL_WC_EXPERIENSE, " ", "warcry of training", 10, 10, 10,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//239
	initSpell(SPELL_WC_LUCK, " ", "warcry of luck", 10, 10, 10,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//240
	initSpell(SPELL_WC_PHYSDAMAGE, " ", "warcry of accuracy", 10, 10, 10,
			  POS_FIGHTING, TAR_IGNORE, FALSE, MAG_WARCRY | MAG_GROUPS | MAG_AFFECTS | NPC_AFFECT_NPC, 0, STYPE_MIND);
//241
	initSpell(SPELL_MASS_FAILURE, " ", "gaze of Veles", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_DARK);
//242
	initSpell(SPELL_MASS_NOFLEE, "", "snare", 140, 120, 2,
			  POS_FIGHTING, TAR_IGNORE, MTYPE_NEUTRAL, MAG_MASSES | MAG_AFFECTS | NPC_AFFECT_PC, 2, STYPE_MIND);

	/*
	 * These spells are currently not used, not implemented, and not castable.
	 * Values for the 'breath' spells are filled in assuming a dragon's breath.
	 */



//243 - 247
	initSpell(SPELL_FIRE_BREATH, " ", "fire breath", 0, 0, 0,
			  POS_SITTING, TAR_IGNORE, TRUE, MAG_DAMAGE, 3, STYPE_FIRE);

	initSpell(SPELL_GAS_BREATH, " ", "gas breath", 0, 0, 0,
			  POS_SITTING, TAR_IGNORE, TRUE, MAG_DAMAGE, 3, STYPE_EARTH);

	initSpell(SPELL_FROST_BREATH, " ", "frost breath", 0, 0, 0,
			  POS_SITTING, TAR_IGNORE, TRUE, MAG_DAMAGE, 3, STYPE_AIR);

	initSpell(SPELL_ACID_BREATH, " ", "acid breath", 0, 0, 0,
			  POS_SITTING, TAR_IGNORE, TRUE, MAG_DAMAGE, 3, STYPE_WATER);

	initSpell(SPELL_LIGHTNING_BREATH, " ", "lightning breath",
			  0, 0, 0, POS_SITTING, TAR_IGNORE, TRUE, MAG_DAMAGE, 3, STYPE_DARK);
//248

//351
	initSpell(SPELL_IDENTIFY, "", "identify",
			  0, 0, 0, 0, TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_ROOM | TAR_OBJ_EQUIP, FALSE, MAG_MANUAL, 0, STYPE_MIND);
//352
	initSpell(SPELL_FULL_IDENTIFY, " ", "identify",
			  0, 0, 0, 0, TAR_CHAR_ROOM | TAR_OBJ_INV | TAR_OBJ_ROOM | TAR_OBJ_EQUIP, FALSE, MAG_MANUAL, 0, STYPE_MIND);
//353  dg_affect
	initSpell(SPELL_QUEST, "!!", "!quest spell!",
			  55, 40, 1, POS_FIGHTING, TAR_CHAR_ROOM | TAR_NOT_SELF, MTYPE_NEUTRAL, MAG_MANUAL, 1, STYPE_NEUTRAL);

}


