#include "backstab.h"

#include "fightsystem/pk.h"
#include "fightsystem/fight.h"
#include "fightsystem/common.h"
#include "fightsystem/fight_hit.h"
#include "fightsystem/fight_start.h"
#include "handler.h"
#include "protect.h"
#include "skills_info.h"

using namespace FightSystem;

//    
void do_backstab(CHAR_DATA *ch, char *argument, int/* cmd*/, int/* subcmd*/) {
	if (ch->get_skill(SKILL_BACKSTAB) < 1) {
		send_to_char("   .\r\n", ch);
		return;
	}
	if (ch->haveCooldown(SKILL_BACKSTAB)) {
		send_to_char("   .\r\n", ch);
		return;
	};

	if (ch->ahorse()) {
		send_to_char("   .\r\n", ch);
		return;
	}

	if (GET_POS(ch) < POS_FIGHTING) {
		send_to_char("    .\r\n", ch);
		return;
	}

	one_argument(argument, arg);
	CHAR_DATA *vict = get_char_vis(ch, arg, FIND_CHAR_ROOM);
	if (!vict) {
		send_to_char("    ,   ?\r\n", ch);
		return;
	}

	if (vict == ch) {
		send_to_char(", , !\r\n", ch);
		return;
	}

	if (!GET_EQ(ch, WEAR_WIELD) && (!IS_NPC(ch) || IS_CHARMICE(ch))) {
		send_to_char("     .\r\n", ch);
		return;
	}

	if ((!IS_NPC(ch) || IS_CHARMICE(ch)) && GET_OBJ_VAL(GET_EQ(ch, WEAR_WIELD), 3) != type_pierce) {
		send_to_char("    !\r\n", ch);
		return;
	}

	if (AFF_FLAGGED(ch, EAffectFlag::AFF_STOPRIGHT) || dontCanAct(ch)) {
		send_to_char("     .\r\n", ch);
		return;
	}

	if (vict->get_fighting() && !can_use_feat(ch, THIEVES_STRIKE_FEAT)) {
		send_to_char("     -   !\r\n", ch);
		return;
	}

	if (!may_kill_here(ch, vict, argument))
		return;
	if (!check_pkill(ch, vict, arg))
		return;
	go_backstab(ch, vict);
}

// *********************** BACKSTAB VICTIM
//          
void go_backstab(CHAR_DATA *ch, CHAR_DATA *vict) {

	if (ch->isHorsePrevents())
		return;

	vict = try_protect(vict, ch);

	if (!pk_agro_action(ch, vict))
		return;

	if ((MOB_FLAGGED(vict, MOB_AWARE) && AWAKE(vict)) && !IS_GOD(ch)) {
		act(" ,  $N $u  !", FALSE, vict, nullptr, ch, TO_CHAR);
		act("$n $g    $s!", FALSE, vict, nullptr, ch, TO_VICT);
		act("$n $g  $N1  $s!", FALSE, vict, nullptr, ch, TO_NOTVICT | TO_ARENA_LISTEN);
		set_hit(vict, ch);
		return;
	}

	bool success = false;
	if (PRF_FLAGGED(ch, PRF_TESTER)) {
		SkillRollResult result = MakeSkillTest(ch, SKILL_BACKSTAB, vict);
		success = result.success;
	} else {
		int percent = number(1, skill_info[SKILL_BACKSTAB].difficulty);
		int prob = CalcCurrentSkill(ch, SKILL_BACKSTAB, vict);

		if (can_use_feat(ch, SHADOW_STRIKE_FEAT)) {
			prob = prob + prob * 20 / 100;
		};

		if (vict->get_fighting()) {
			prob = prob * (GET_REAL_DEX(ch) + 50) / 100;
		}

		if (AFF_FLAGGED(ch, EAffectFlag::AFF_HIDE)) {
			prob += 5;
		}
		if (GET_MOB_HOLD(vict)) {
			prob = prob * 5 / 4;
		}
		if (GET_GOD_FLAG(vict, GF_GODSCURSE)) {
			prob = percent;
		}
		if (GET_GOD_FLAG(vict, GF_GODSLIKE) || GET_GOD_FLAG(ch, GF_GODSCURSE)) {
			prob = 0;
		}
		success = percent <= prob;
		SendSkillBalanceMsg(ch, skill_info[SKILL_BACKSTAB].name, percent, prob, success);
	}

	TrainSkill(ch, SKILL_BACKSTAB, success, vict);
	if (!success) {
		Damage dmg(SkillDmg(SKILL_BACKSTAB), ZERO_DMG, PHYS_DMG);
		dmg.process(ch, vict);
	} else {
		hit(ch, vict, SKILL_BACKSTAB, FightSystem::MAIN_HAND);
	}
	setSkillCooldownInFight(ch, SKILL_GLOBAL_COOLDOWN, 1);
	setSkillCooldownInFight(ch, SKILL_BACKSTAB, 2);
}

// vim: ts=4 sw=4 tw=0 noet syntax=cpp :
