#include "chopoff.h"

#include "fightsystem/pk.h"
#include "fightsystem/common.h"
#include "fightsystem/fight_hit.h"
#include "fightsystem/fight_start.h"
#include "random.h"
#include "screen.h"
#include "skills_info.h"

using namespace FightSystem;

// ************************* CHOPOFF PROCEDURES
void go_chopoff(CHAR_DATA *ch, CHAR_DATA *vict) {
	if (dontCanAct(ch)) {
		send_to_char("     .\r\n", ch);
		return;
	}

	if (PRF_FLAGS(ch).get(PRF_IRON_WIND)) {
		send_to_char("        !\r\n", ch);
		return;
	}

	if (ch->isHorsePrevents())
		return;

	if ((GET_POS(vict) < POS_FIGHTING)) {
		if (number(1, 100) < ch->get_skill(SKILL_CHOPOFF)) {
			send_to_char("   ,   .\r\n", ch);
			//set_wait(ch, 1, FALSE);
			//setSkillCooldown(ch, SKILL_CHOPOFF, 1);
			ch->setSkillCooldown(SKILL_CHOPOFF, PULSE_VIOLENCE / 6);
			return;
		}
	}

	if (!pk_agro_action(ch, vict))
		return;

	int percent = number(1, skill_info[SKILL_CHOPOFF].difficulty);
	int prob = CalcCurrentSkill(ch, SKILL_CHOPOFF, vict);

	if (check_spell_on_player(ch, SPELL_WEB)) {
		prob /= 3;
	}
	if (GET_GOD_FLAG(ch, GF_GODSLIKE)
		|| GET_MOB_HOLD(vict)
		|| GET_GOD_FLAG(vict, GF_GODSCURSE)) {
		prob = percent;
	}

	if (GET_GOD_FLAG(ch, GF_GODSCURSE) ||
		GET_GOD_FLAG(vict, GF_GODSLIKE) ||
		vict->ahorse() || GET_POS(vict) < POS_FIGHTING || MOB_FLAGGED(vict, MOB_NOTRIP) || IS_IMMORTAL(vict))
		prob = 0;

	bool success = percent <= prob;
	TrainSkill(ch, SKILL_CHOPOFF, success, vict);
	SendSkillBalanceMsg(ch, skill_info[SKILL_CHOPOFF].name, percent, prob, success);
	if (!success) {
		sprintf(buf, "%s   $N3,   ...%s", CCWHT(ch, C_NRM), CCNRM(ch, C_NRM));
		act(buf, FALSE, ch, 0, vict, TO_CHAR);
		act("$n $u  ,  $g $g.", FALSE, ch, 0, vict, TO_VICT);
		act("$n $u  $N3,  $g $g.", TRUE, ch, 0, vict, TO_NOTVICT | TO_ARENA_LISTEN);
		GET_POS(ch) = POS_SITTING;
		prob = 3;
		if (can_use_feat(ch, EVASION_FEAT)) {
			AFFECT_DATA<EApplyLocation> af;
			af.type = SPELL_EXPEDIENT;
			af.location = EApplyLocation::APPLY_PR;
			af.modifier = 50;
			af.duration = pc_duration(ch, 3, 0, 0, 0, 0);
			af.battleflag = AF_BATTLEDEC | AF_PULSEDEC;
			affect_join(ch, af, FALSE, FALSE, FALSE, FALSE);
			af.location = EApplyLocation::APPLY_AR;
			affect_join(ch, af, FALSE, FALSE, FALSE, FALSE);
			af.location = EApplyLocation::APPLY_MR;
			affect_join(ch, af, FALSE, FALSE, FALSE, FALSE);
			send_to_char(ch,
						 "%s   ,    %s.%s\r\n",
						 CCIGRN(ch, C_NRM),
						 GET_PAD(vict, 1),
						 CCNRM(ch, C_NRM));
			act("$n $u  ,    .", FALSE, ch, 0, vict, TO_VICT);
			act("$n $u  ,    $N1.", TRUE, ch, 0, vict, TO_NOTVICT | TO_ARENA_LISTEN);
		}
	} else {
		send_to_char(ch,
					 "%s  ,   %s  .%s\r\n",
					 CCIBLU(ch, C_NRM),
					 GET_PAD(vict, 3),
					 CCNRM(ch, C_NRM));
		act("$n  $q ,   .", FALSE, ch, 0, vict, TO_VICT);
		act("$n  $q $N3,  $S  .", TRUE, ch, 0, vict, TO_NOTVICT | TO_ARENA_LISTEN);
		set_wait(vict, 3, FALSE);

		if (ch->isInSameRoom(vict)) {
			GET_POS(vict) = POS_SITTING;
		}

		if (IS_HORSE(vict) && vict->get_master()->ahorse()) {
			vict->drop_from_horse();
		}
		prob = 1;
	}

	appear(ch);
	if (IS_NPC(vict) && CAN_SEE(vict, ch) && vict->have_mind() && vict->get_wait() <= 0) {
		set_hit(vict, ch);
	}

	if (!success) {
		set_wait(ch, prob, FALSE);
	} else {
		setSkillCooldownInFight(ch, SKILL_CHOPOFF, prob);
		setSkillCooldownInFight(ch, SKILL_GLOBAL_COOLDOWN, 1);
	}
}

void do_chopoff(CHAR_DATA *ch, char *argument, int/* cmd*/, int/* subcmd*/) {
	if (ch->get_skill(SKILL_CHOPOFF) < 1) {
		send_to_char("   .\r\n", ch);
		return;
	}
	if (ch->haveCooldown(SKILL_CHOPOFF)) {
		send_to_char("   .\r\n", ch);
		return;
	};

	if (ch->ahorse()) {
		send_to_char("   .\r\n", ch);
		return;
	}

	CHAR_DATA *vict = findVictim(ch, argument);
	if (!vict) {
		send_to_char("   ?\r\n", ch);
		return;
	}

	if (vict == ch) {
		send_to_char("    <>.\r\n", ch);
		return;
	}

	if (!may_kill_here(ch, vict, argument))
		return;
	if (!check_pkill(ch, vict, arg))
		return;

	if (IS_IMPL(ch) || !ch->get_fighting())
		go_chopoff(ch, vict);
	else if (isHaveNoExtraAttack(ch)) {
		if (!IS_NPC(ch))
			act(".    $N3.", FALSE, ch, 0, vict, TO_CHAR);
		ch->set_extra_attack(EXTRA_ATTACK_CHOPOFF, vict);
	}
}
