//
// Created by ubuntu on 03/09/20.
//

#include "expendientcut.h"
#include "fightsystem/common.h"
#include "fightsystem/fight.h"
#include "fightsystem/fight_hit.h"
#include "fightsystem/pk.h"
#include "skills/protect.h"

using namespace FightSystem;

ESkill ExpedientWeaponSkill(CHAR_DATA *ch) {
	ESkill skill = SKILL_PUNCH;

	if (GET_EQ(ch, WEAR_WIELD) && (GET_OBJ_TYPE(GET_EQ(ch, WEAR_WIELD)) == CObjectPrototype::ITEM_WEAPON)) {
		skill = static_cast<ESkill>GET_OBJ_SKILL(GET_EQ(ch, WEAR_WIELD));
	} else if (GET_EQ(ch, WEAR_BOTHS) && (GET_OBJ_TYPE(GET_EQ(ch, WEAR_BOTHS)) == CObjectPrototype::ITEM_WEAPON)) {
		skill = static_cast<ESkill>GET_OBJ_SKILL(GET_EQ(ch, WEAR_BOTHS));
	} else if (GET_EQ(ch, WEAR_HOLD) && (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD)) == CObjectPrototype::ITEM_WEAPON)) {
		skill = static_cast<ESkill>GET_OBJ_SKILL(GET_EQ(ch, WEAR_HOLD));
	};

	return skill;
}
int GetExpedientKeyParameter(CHAR_DATA *ch, ESkill skill) {
	switch (skill) {
		case SKILL_PUNCH:
		case SKILL_CLUBS:
		case SKILL_AXES:
		case SKILL_BOTHHANDS:
		case SKILL_SPADES:return ch->get_str();
			break;
		case SKILL_LONGS:
		case SKILL_SHORTS:
		case SKILL_NONSTANDART:
		case SKILL_BOWS:
		case SKILL_PICK:return ch->get_dex();
			break;
		default:return ch->get_str();
	}
}
int ParameterBonus(int parameter) {
	return ((parameter - 20) / 4);
}
int ExpedientRating(CHAR_DATA *ch, ESkill skill) {
	return floor(ch->get_skill(skill) / 2.00 + ParameterBonus(GetExpedientKeyParameter(ch, skill)));
}
int ExpedientCap(CHAR_DATA *ch, ESkill skill) {
	if (!IS_NPC(ch)) {
		return floor(1.33 * (CalcSkillRemortCap(ch) / 2.00 + ParameterBonus(GetExpedientKeyParameter(ch, skill))));
	} else {
		return floor(1.33 * ((kSkillCapOnZeroRemort + 5 * MAX(0, GET_REAL_LEVEL(ch) - 30) / 2.00
			+ ParameterBonus(GetExpedientKeyParameter(ch, skill)))));
	}
}
int DegreeOfSuccess(int roll, int rating) {
	return ((rating - roll) / 5);
}

bool CheckExpedientSuccess(CHAR_DATA *ch, CHAR_DATA *victim) {
	ESkill DoerSkill = ExpedientWeaponSkill(ch);
	int DoerRating = ExpedientRating(ch, DoerSkill);
	int DoerCap = ExpedientCap(ch, DoerSkill);
	int DoerRoll = dice(1, DoerCap);
	int DoerSuccess = DegreeOfSuccess(DoerRoll, DoerRating);

	ESkill VictimSkill = ExpedientWeaponSkill(victim);
	int VictimRating = ExpedientRating(victim, VictimSkill);
	int VictimCap = ExpedientCap(victim, VictimSkill);
	int VictimRoll = dice(1, VictimCap);
	int VictimSuccess = DegreeOfSuccess(VictimRoll, VictimRating);

	if ((DoerRoll <= DoerRating) && (VictimRoll > VictimRating))
		return true;
	if ((DoerRoll > DoerRating) && (VictimRoll <= VictimRating))
		return false;
	if ((DoerRoll > DoerRating) && (VictimRoll > VictimRating))
		return CheckExpedientSuccess(ch, victim);

	if (DoerSuccess > VictimSuccess)
		return true;
	if (DoerSuccess < VictimSuccess)
		return false;

	if (ParameterBonus(GetExpedientKeyParameter(ch, DoerSkill))
		> ParameterBonus(GetExpedientKeyParameter(victim, VictimSkill)))
		return true;
	if (ParameterBonus(GetExpedientKeyParameter(ch, DoerSkill))
		< ParameterBonus(GetExpedientKeyParameter(victim, VictimSkill)))
		return false;

	if (DoerRoll < VictimRoll)
		return true;
	if (DoerRoll > VictimRoll)
		return true;

	return CheckExpedientSuccess(ch, victim);
}

void ApplyNoFleeAffect(CHAR_DATA *ch, int duration) {
	//  ,  -    2    ,     
	//-, -   ,       1
	AFFECT_DATA<EApplyLocation> Noflee;
	Noflee.type = SPELL_BATTLE;
	Noflee.bitvector = to_underlying(EAffectFlag::AFF_NOFLEE);
	Noflee.location = EApplyLocation::APPLY_NONE;
	Noflee.modifier = 0;
	Noflee.duration = pc_duration(ch, duration, 0, 0, 0, 0);;
	Noflee.battleflag = AF_BATTLEDEC | AF_PULSEDEC;
	affect_join(ch, Noflee, TRUE, FALSE, TRUE, FALSE);

	AFFECT_DATA<EApplyLocation> Battle;
	Battle.type = SPELL_BATTLE;
	Battle.bitvector = to_underlying(EAffectFlag::AFF_EXPEDIENT);
	Battle.location = EApplyLocation::APPLY_NONE;
	Battle.modifier = 0;
	Battle.duration = pc_duration(ch, duration, 0, 0, 0, 0);;
	Battle.battleflag = AF_BATTLEDEC | AF_PULSEDEC;
	affect_join(ch, Battle, TRUE, FALSE, TRUE, FALSE);

	send_to_char("    .\r\n", ch);
}

void go_cut_shorts(CHAR_DATA *ch, CHAR_DATA *vict) {

	if (dontCanAct(ch)) {
		send_to_char("     .\r\n", ch);
		return;
	}

	if (AFF_FLAGGED(ch, EAffectFlag::AFF_EXPEDIENT)) {
		send_to_char("       .\r\n", ch);
		return;
	}

	vict = try_protect(vict, ch);

	if (!CheckExpedientSuccess(ch, vict)) {
		act("    ,   $N3.", FALSE, ch, 0, vict, TO_CHAR);
		Damage dmg(SkillDmg(SKILL_SHORTS), ZERO_DMG, PHYS_DMG);
		dmg.process(ch, vict);
		ApplyNoFleeAffect(ch, 2);
		return;
	}

	act("$n $g      $q  .", FALSE, ch, 0, vict, TO_VICT);
	act("$n $g  ,    $N1.",
		TRUE, ch, 0, vict, TO_NOTVICT | TO_ARENA_LISTEN);
	hit(ch, vict, ESkill::SKILL_UNDEF, FightSystem::MAIN_HAND);
	hit(ch, vict, ESkill::SKILL_UNDEF, FightSystem::OFF_HAND);

	AFFECT_DATA<EApplyLocation> AffectImmunPhysic;
	AffectImmunPhysic.type = SPELL_EXPEDIENT;
	AffectImmunPhysic.location = EApplyLocation::APPLY_PR;
	AffectImmunPhysic.modifier = 100;
	AffectImmunPhysic.duration = pc_duration(ch, 3, 0, 0, 0, 0);
	AffectImmunPhysic.battleflag = AF_BATTLEDEC | AF_PULSEDEC;
	affect_join(ch, AffectImmunPhysic, FALSE, FALSE, FALSE, FALSE);
	AFFECT_DATA<EApplyLocation> AffectImmunMagic;
	AffectImmunMagic.type = SPELL_EXPEDIENT;
	AffectImmunMagic.location = EApplyLocation::APPLY_MR;
	AffectImmunMagic.modifier = 100;
	AffectImmunMagic.duration = pc_duration(ch, 3, 0, 0, 0, 0);
	AffectImmunMagic.battleflag = AF_BATTLEDEC | AF_PULSEDEC;
	affect_join(ch, AffectImmunMagic, FALSE, FALSE, FALSE, FALSE);

	ApplyNoFleeAffect(ch, 3);
}

void SetExtraAttackCutShorts(CHAR_DATA *ch, CHAR_DATA *victim) {
	if (!isHaveNoExtraAttack(ch)) {
		return;
	}

	if (!pk_agro_action(ch, victim)) {
		return;
	}

	if (!ch->get_fighting()) {
		act("  ,     $N3,  \"\".", FALSE, ch, 0, victim, TO_CHAR);
		set_fighting(ch, victim);
		ch->set_extra_attack(EXTRA_ATTACK_CUT_SHORTS, victim);
	} else {
		act(".    $N3.", FALSE, ch, 0, victim, TO_CHAR);
		ch->set_extra_attack(EXTRA_ATTACK_CUT_SHORTS, victim);
	}
}

void SetExtraAttackCutPick(CHAR_DATA *ch, CHAR_DATA *victim) {
	if (!isHaveNoExtraAttack(ch)) {
		return;
	}
	if (!pk_agro_action(ch, victim)) {
		return;
	}

	if (!ch->get_fighting()) {
		act("         $N1.", FALSE, ch, 0, victim, TO_CHAR);
		set_fighting(ch, victim);
		ch->set_extra_attack(EXTRA_ATTACK_CUT_PICK, victim);
	} else {
		act(".    $N3.", FALSE, ch, 0, victim, TO_CHAR);
		ch->set_extra_attack(EXTRA_ATTACK_CUT_PICK, victim);
	}
}

ESkill GetExpedientCutSkill(CHAR_DATA *ch) {
	ESkill skill = SKILL_INVALID;

	if (GET_EQ(ch, WEAR_WIELD) && GET_EQ(ch, WEAR_HOLD)) {
		skill = static_cast<ESkill>GET_OBJ_SKILL(GET_EQ(ch, WEAR_WIELD));
		if (skill != GET_OBJ_SKILL(GET_EQ(ch, WEAR_HOLD))) {
			send_to_char("          !\r\n", ch);
			return SKILL_INVALID;
		}
	} else if (GET_EQ(ch, WEAR_BOTHS)) {
		skill = static_cast<ESkill>GET_OBJ_SKILL(GET_EQ(ch, WEAR_BOTHS));
	} else {
		send_to_char("            .\r\n", ch);
		return SKILL_INVALID;
	}

	if (!can_use_feat(ch, FindWeaponMasterBySkill(skill)) && !IS_IMPL(ch)) {
		send_to_char("        .\r\n", ch);
		return SKILL_INVALID;
	}

	return skill;
}

void do_expedient_cut(CHAR_DATA *ch, char *argument, int/* cmd*/, int /*subcmd*/) {

	ESkill skill;

	if (IS_NPC(ch) || (!can_use_feat(ch, EXPEDIENT_CUT_FEAT) && !IS_IMPL(ch))) {
		send_to_char("    .\r\n", ch);
		return;
	}

	if (ch->ahorse()) {
		send_to_char("   .\r\n", ch);
		return;
	}

	if (GET_POS(ch) < POS_FIGHTING) {
		send_to_char("    .\r\n", ch);
		return;
	}

	if (!isHaveNoExtraAttack(ch)) {
		return;
	}

	if (AFF_FLAGGED(ch, EAffectFlag::AFF_STOPRIGHT) || AFF_FLAGGED(ch, EAffectFlag::AFF_STOPFIGHT)
		|| AFF_FLAGGED(ch, EAffectFlag::AFF_MAGICSTOPFIGHT)) {
		send_to_char("     .\r\n", ch);
		return;
	}

	CHAR_DATA *vict = findVictim(ch, argument);
	if (!vict) {
		send_to_char("   ?\r\n", ch);
		return;
	}

	if (vict == ch) {
		send_to_char("  ? -,     ,   !\r\n", ch);
		return;
	}
	if (ch->get_fighting() && vict->get_fighting() != ch) {
		act("$N    ,   $S.", FALSE, ch, 0, vict, TO_CHAR);
		return;
	}

	if (!may_kill_here(ch, vict, argument))
		return;
	if (!check_pkill(ch, vict, arg))
		return;

	skill = GetExpedientCutSkill(ch);
	if (skill == SKILL_INVALID)
		return;

	switch (skill) {
		case SKILL_SHORTS:SetExtraAttackCutShorts(ch, vict);
			break;
		case SKILL_SPADES:SetExtraAttackCutShorts(ch, vict);
			break;
		case SKILL_LONGS:
		case SKILL_BOTHHANDS:
			send_to_char("  (     ) -  .   .\r\n",
						 ch);
			break;
		default:send_to_char("      .\r\n", ch);
	}

}
