#include "townportal.h"

#include "modify.h"
#include "handler.h"
#include "magic/magic_rooms.h"
#include "fightsystem/pk.h"

namespace OneWayPortal {

//    < ,  >
std::unordered_map<room_vnum /*to*/, ROOM_DATA * /*from*/> portal_list;

/**
*    
* \param to_room -   
* \param from_room -  
*/
void add(ROOM_DATA *to_room, ROOM_DATA *from_room) {
	portal_list.emplace(to_room->room_vn, from_room);
}

/**
*    
* \param to_room -   
*/
void remove(ROOM_DATA *to_room) {
	const auto it = portal_list.find(to_room->room_vn);
	if (it != portal_list.end())
		portal_list.erase(it);
}

/**
*      
* \param to_room -   
* \return    
*/
ROOM_DATA *get_from_room(ROOM_DATA *to_room) {

	const auto it = portal_list.find(to_room->room_vn);
	if (it != portal_list.end())
		return it->second;

	return nullptr;
}

} // namespace OneWayPortal

void spell_townportal(CHAR_DATA *ch, char *arg) {
	int gcount = 0, cn = 0, ispr = 0;
	bool has_label_portal = false;
	struct timed_type timed;
	char *nm;
	struct char_portal_type *tmp;
	struct portals_list_type *port;
	struct portals_list_type label_port;
	ROOM_DATA *label_room;

	port = get_portal(-1, arg);

	//  , ,       
	if (!port && name_cmp(ch, arg)) {

		label_room = RoomSpells::findAffectedRoom(GET_ID(ch), SPELL_RUNE_LABEL);
		if (label_room) {
			label_port.vnum = label_room->room_vn;
			label_port.level = 1;
			port = &label_port;
			has_label_portal = true;
		}
	}
	if (port && (has_char_portal(ch, port->vnum) || has_label_portal)) {
		if (timed_by_skill(ch, SKILL_TOWNPORTAL)) {
			send_to_char("      .\r\n", ch);
			return;
		}

		if (find_portal_by_vnum(GET_ROOM_VNUM(ch->in_room))) {
			send_to_char("      .\r\n", ch);
			return;
		}

		if (RoomSpells::isRoomAffected(world[ch->in_room], SPELL_RUNE_LABEL)) {
			send_to_char("       !\r\n", ch);
			return;
		}

		if (ROOM_FLAGGED(ch->in_room, ROOM_NOMAGIC) && !IS_GRGOD(ch)) {
			send_to_char("       .\r\n", ch);
			act(" $n1      .", FALSE, ch, 0, 0, TO_ROOM);
			return;
		}
		// 
		if (world[ch->in_room]->portal_time) {
			if (world[world[ch->in_room]->portal_room]->portal_room == ch->in_room
				&& world[world[ch->in_room]->portal_room]->portal_time) {
				decay_portal(world[ch->in_room]->portal_room);
			}
			decay_portal(ch->in_room);
		}

		//     rnum //
		ImproveSkill(ch, SKILL_TOWNPORTAL, 1, NULL);
		ROOM_DATA *from_room = world[ch->in_room];
		from_room->portal_room = real_room(port->vnum);
		from_room->portal_time = 1;
		from_room->pkPenterUnique = 0;
		OneWayPortal::add(world[from_room->portal_room], from_room);
		act("    .", FALSE, ch, 0, 0, TO_CHAR);
		act("$n $g    ,    ...", FALSE, ch, 0, 0, TO_ROOM);
		act("    .", FALSE, ch, 0, 0, TO_ROOM);
		if (!IS_IMMORTAL(ch)) {
			timed.skill = SKILL_TOWNPORTAL;
			// timed.time -  unsigned char,         255  
			int modif = ch->get_skill(SKILL_TOWNPORTAL) / 7 + number(1, 5);
			timed.time = MAX(1, 25 - modif);
			timed_to_char(ch, &timed);
		}
		return;
	}

	//    //
	gcount = sprintf(buf2 + gcount, "   :\r\n");
	for (tmp = GET_PORTALS(ch); tmp; tmp = tmp->next) {
		nm = find_portal_by_vnum(tmp->vnum);
		if (nm) {
			gcount += sprintf(buf2 + gcount, "%11s", nm);
			cn++;
			ispr++;
			if (cn == 3) {
				gcount += sprintf(buf2 + gcount, "\r\n");
				cn = 0;
			}
		}
	}
	if (cn)
		gcount += sprintf(buf2 + gcount, "\r\n");
	if (!ispr) {
		gcount += sprintf(buf2 + gcount, "          .\r\n");
	} else {
		gcount += sprintf(buf2 + gcount, "        - %d.\r\n", ispr);
	}
	gcount += sprintf(buf2 + gcount, "          - %d.\r\n", MAX_PORTALS(ch));

	page_string(ch->desc, buf2, 1);
}

void do_townportal(CHAR_DATA *ch, char *argument, int/* cmd*/, int/* subcmd*/) {

	struct char_portal_type *tmp, *dlt = NULL;
	char arg2[MAX_INPUT_LENGTH];
	int vnum = 0;

	if (IS_NPC(ch) || !ch->get_skill(SKILL_TOWNPORTAL)) {
		send_to_char("    .\r\n", ch);
		return;
	}

	two_arguments(argument, arg, arg2);
	if (!str_cmp(arg, "")) {
		vnum = find_portal_by_word(arg2);
		for (tmp = GET_PORTALS(ch); tmp; tmp = tmp->next) {
			if (tmp->vnum == vnum) {
				if (dlt) {
					dlt->next = tmp->next;
				} else {
					GET_PORTALS(ch) = tmp->next;
				}
				free(tmp);
				sprintf(buf, "  ,    '&R%s&n'.\r\n", arg2);
				send_to_char(buf, ch);
				break;
			}
			dlt = tmp;
		}
		return;
	}

	spell_townportal(ch, arg);
}
