#include "skills_info.h"

const char *kUnusedSkillName = "!UNUSED!";
const char *kUnidentifiedSkillName = "!UNIDENTIFIED!";
const byte kDefaultMinPosition = 0;
const int kDefaultFailPercent = 200;
const unsigned short kDefaultCap = 200;

struct SkillInfoType skill_info[MAX_SKILL_NUM + 1];

const char *skill_name(int num);
void InitSingleSkill(int skill, const char *name, const char *short_name,
					 int save_type, int difficulty, unsigned short cap);
void InitSkills();

const char *skill_name(int num) {
	if (num > 0 && num <= MAX_SKILL_NUM) {
		return (skill_info[num].name);
	} else {
		if (num == -1) {
			return kUnusedSkillName;
		} else {
			return kUnidentifiedSkillName;
		}
	}
}

void InitUnusedSkill(int skill) {
	int i, j;

	for (i = 0; i < NUM_PLAYER_CLASSES; i++) {
		for (j = 0; j < NUM_KIN; j++) {
			skill_info[skill].min_remort[i][j] = MAX_REMORT;
			skill_info[skill].min_level[i][j] = 0;
			skill_info[skill].k_improve[i][j] = 0;
		}
	}
	skill_info[skill].min_position = kDefaultMinPosition;
	skill_info[skill].save_type = SAVING_REFLEX;
	skill_info[skill].difficulty = kDefaultFailPercent;
	skill_info[skill].cap = kDefaultCap;
	skill_info[skill].name = kUnusedSkillName;
}

void InitSingleSkill(int skill, const char *name, const char *short_name,
					 int save_type, int difficulty, unsigned short cap) {
	int i, j;
	for (i = 0; i < NUM_PLAYER_CLASSES; i++) {
		for (j = 0; j < NUM_KIN; j++) {
			skill_info[skill].min_remort[i][j] = MAX_REMORT;
			skill_info[skill].min_level[i][j] = 0;
			skill_info[skill].k_improve[i][j] = 0;
		}
	}
	skill_info[skill].min_position = 0;
	skill_info[skill].name = name;
	skill_info[skill].shortName = short_name;
	skill_info[skill].save_type = save_type;
	skill_info[skill].difficulty = difficulty;
	skill_info[skill].cap = cap;
}

void InitSkills() {

	for (int i = 0; i <= MAX_SKILL_NUM; i++) {
		InitUnusedSkill(i);
	}

	InitSingleSkill(SKILL_GLOBAL_COOLDOWN, "!global cooldown", "", SAVING_REFLEX, 1, 1);
	InitSingleSkill(SKILL_BACKSTAB, "", "", SAVING_REFLEX, 25, 1000);
	InitSingleSkill(SKILL_BASH, "", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_HIDE, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_KICK, "", "", SAVING_STABILITY, 0, 200);
	InitSingleSkill(SKILL_PICK_LOCK, "", "", SAVING_REFLEX, 120, 200);
	InitSingleSkill(SKILL_PUNCH, " ", "", SAVING_REFLEX, 100, 1000);
	InitSingleSkill(SKILL_RESCUE, "", "", SAVING_REFLEX, 130, 1000);
	InitSingleSkill(SKILL_SNEAK, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_STEAL, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_TRACK, "", "", SAVING_REFLEX, 100, 1000);
	InitSingleSkill(SKILL_PARRY, "", "", SAVING_REFLEX, 120, 1000);
	InitSingleSkill(SKILL_BLOCK, " ", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_TOUCH, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_PROTECT, "", "", SAVING_REFLEX, 120, 200);
	InitSingleSkill(SKILL_BOTHHANDS, "", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_LONGS, " ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_SPADES, "  ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_SHORTS, " ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_BOWS, "", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_CLUBS, "  ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_PICK, " ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_NONSTANDART, " ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_AXES, "", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_SATTACK, "  ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_LOOKING, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_HEARING, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_DISARM, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_HEAL, "!heal!", "Hl", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_MORPH, "", "", SAVING_REFLEX, 150, 200);
	InitSingleSkill(SKILL_ADDSHOT, " ", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_CAMOUFLAGE, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_DEVIATE, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_CHOPOFF, "", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_REPAIR, "", "", SAVING_REFLEX, 180, 200);
	InitSingleSkill(SKILL_COURAGE, "", "", SAVING_REFLEX, 100, 1000);
	InitSingleSkill(SKILL_IDENTIFY, "", "", SAVING_REFLEX, 100, 1000);
	InitSingleSkill(SKILL_LOOK_HIDE, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_UPGRADE, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_ARMORED, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_DRUNKOFF, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_AID, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_FIRE, " ", "", SAVING_REFLEX, 160, 1000);
	InitSingleSkill(SKILL_SHIT, "  ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_MIGHTHIT, " ", "", SAVING_STABILITY, 200, 200);
	InitSingleSkill(SKILL_STUPOR, "", "", SAVING_STABILITY, 200, 200);
	InitSingleSkill(SKILL_POISONED, "", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_LEADERSHIP, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_PUNCTUAL, " ", "", SAVING_CRITICAL, 110, 200);
	InitSingleSkill(SKILL_AWAKE, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_SENSE, "", "", SAVING_REFLEX, 160, 200);
	InitSingleSkill(SKILL_HORSE, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_HIDETRACK, " ", "", SAVING_REFLEX, 120, 200);
	InitSingleSkill(SKILL_RELIGION, "!  !", "Error", SAVING_REFLEX, 1, 1);
	InitSingleSkill(SKILL_MAKEFOOD, "", "", SAVING_REFLEX, 120, 200);
	InitSingleSkill(SKILL_MULTYPARRY, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_TRANSFORMWEAPON, "", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_THROW, "", "", SAVING_REFLEX, 150, 1000);
	InitSingleSkill(SKILL_MAKE_BOW, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_MAKE_WEAPON, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_MAKE_ARMOR, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_MAKE_WEAR, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_MAKE_JEWEL, " ", "", SAVING_REFLEX, 140, 200);
	InitSingleSkill(SKILL_MANADRAIN, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_NOPARRYHIT, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_TOWNPORTAL, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_DIG, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_INSERTGEM, "", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_WARCRY, " ", "", SAVING_REFLEX, 100, 200);
	InitSingleSkill(SKILL_TURN_UNDEAD, " ", "", SAVING_REFLEX, 100, 1000);
	InitSingleSkill(SKILL_IRON_WIND, " ", "", SAVING_REFLEX, 150, 1000);
	InitSingleSkill(SKILL_STRANGLE, "", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_AIR_MAGIC, " ", "", SAVING_REFLEX, 200, 1000);
	InitSingleSkill(SKILL_FIRE_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_WATER_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_EARTH_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_LIGHT_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_DARK_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_MIND_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_LIFE_MAGIC, " ", "", SAVING_REFLEX, 1000, 1000);
	InitSingleSkill(SKILL_STUN, "", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_MAKE_AMULET, " ", "", SAVING_REFLEX, 200, 200);
	InitSingleSkill(SKILL_INDEFINITE, "!", "Error", SAVING_REFLEX, 1, 1);
}
